"""Control server.

This server runs on the Beaglebone Black of each eno node, exposing various
endpoints so data can be sent to the node and so we can read data back:

  * GET / to show node info
  * POST /sms to send a message
  * POST /call to initiate a call
  * POST /hangup to hangup an in-progress call
  * POST /data to access a webpage
  * GET /log/<activity> shows logs for calls, SMS and data usage
  * DELETE /log/<activity> to clear usage logs
"""

import logging

import Adafruit_BBIO.UART as UART
import flask
from gsmmodem.modem import GsmModem


PORT = 'UART1'
DEVICE = '/dev/ttyO1'
BAUD = 9600


# Setup the logger, a serial port, and the flask app.
logging.basicConfig(format='%(levelname)s: %(message)s', level=logging.DEBUG)
UART.setup(PORT)
app = flask.Flask(__name__)


@app.route('/')
def index():
  """Get Node info."""
  modem = GsmModem(DEVICE, BAUD)
  modem.connect()
  try:
    return flask.jsonify(
      imsi=modem.imsi,
      model=modem.model,
      manufacturer=modem.manufacturer,
      network_name=modem.networkName,
      signal_strength=modem.signalStrength,
    )
  finally:
    modem.close()


@app.route('/sms', methods=['POST'])
def sms():
  """Send an SMS.

  Expected POST data: phone_number, message
  """
  phone_number = flask.request.form['phone_number']
  message = flask.request.form['message']
  modem = GsmModem(DEVICE, BAUD)
  modem.connect()
  modem.sendSms(phone_number, message)
  modem.close()
  return ''


@app.route('/call', methods=['POST'])
def call():
  """Initiate a call.

  Expected POST data: phone_number, hangup_immediately
  """


@app.route('/hangup', methods=['POST'])
def hangup():
  """Hangup any in-progress calls.

  Expected POST data: (none)
  """


@app.route('/data', methods=['POST'])
def data():
  """Access a webpage."""


@app.route('/log/<activity>', methods=['GET', 'DELETE'])
def log(activity):
  """Call, SMS and data usage history.

  GET to view the data or DELETE to erase it.
  """
  # Validate.
  if activity not in ('sms', 'call', 'data'):
    return '', 400
  # Setup the GSM modem.
  modem = GsmModem(DEVICE, BAUD)
  modem.connect()
  # View the SMS log.
  if flask.request.method == 'GET' and activity == 'sms':
    messages = []
    for message in modem.listStoredSms():
      messages.append({
        'time': message.time,
        'number': message.number,
        'text': message.text,
      })
    modem.close()
    return flask.jsonify(
      messages=messages,
    )
  # Clear the SMS log.
  elif flask.request.method == 'DELETE' and activity == 'sms':
    modem.listStoredSms(delete=True)
    return ''



"""Abstraction of a remote eno hardware node."""

import requests
import yaml


class Node(object):
  """Representation of a remote eno hardware node.

  This class is used on the testing machine to control a remote eno node.  The
  hardware itself does not use this class, it instead uses the control server.
  """

  def __init__(self, **kwargs):
    self.name = kwargs.pop('name', '')
    self.ip_address = kwargs.pop('ip_address', '')
    self.sim = kwargs.pop('sim', '')
    self.phone_number = kwargs.pop('phone_number', '')

  def sms(self, phone_number, message):
    """Send an SMS."""
    data = {
      'phone_number': phone_number,
      'message': message,
    }
    endpoint = '%s/sms' % self.ip_address
    response = requests.post(endpoint, data=data)
    if response.status_code != 200:
      raise ValueError

  def call(self, phone_number, **kwargs):
    """Make a call.

    Args:
      phone_number: the number to call

    Kwargs:
      hangup_immediately: whether to hangup immediately after the call is
                          answered (default is True)
    """
    hangup_immediately = kwargs.get('hangup_immediately', True)
    data = {
      'phone_number': phone_number,
      'hangup_immediately': hangup_immediately,
    }
    endpoint = '%s/call' % self.ip_address
    response = requests.post(endpoint, data=data)
    if response.status_code != 200:
      raise ValueError

  def hangup(self):
    """Terminates any ongoing call."""
    endpoint = '%s/hangup' % self.ip_address
    response = requests.post(endpoint)
    if response.status_code != 200:
      raise ValueError

  def data(self, target):
    """Use data services.

    Args:
      target: will send an HTTP GET to this address
    """
    data = {
      'target': target,
    }
    endpoint = '%s/data' % self.ip_address
    response = requests.post(endpoint, data=data)
    if response.status_code != 200:
      raise ValueError

  def wait_for_activity(self, activity, **kwargs):
    """Block until some activity completes.

    This could mean waiting until an SMS is received, waiting until a call is
    received or blocking until data is returned from a website.

    Args:
      activity: one of sms, call or data

    Kwargs:
      text: blocks until an SMS with this particular text is received
      sender: blocks until a call from this number is received
      target: blocks until data from this target is received
      timeout: the max amount of time to block (default: 10s)
    """

  def get_log(self, activity):
    """Gets info from an activity log.

    Args:
      activity: one of sms, call or data
    """
    if activity not in ('sms', 'call', 'data'):
      raise ValueError
    endpoint = '%s/log/%s' % (self.ip_address, activity)
    response = requests.get(endpoint)
    if response.status_code != 200:
      raise ValueError
    return response.json()

  def reset_log(self, activity):
    """Resets an activity log.

    Args:
      activity: one of sms, call or data
    """
    if activity not in ('sms', 'call', 'data'):
      raise ValueError
    endpoint = '%s/log/%s' % (self.ip_address, activity)
    response = requests.delete(endpoint)
    if response.status_code != 200:
      raise ValueError


def get_node(name):
  """Shortcut method to get an eno node's data.

  Args:
    name: the name of the node

  Returns a Node instance.
  """
  with open('~/.enorc') as config_file:
    config_data = yaml.load(config_file.read())
  # Verify that the requested name is actually in the config file.
  names = [n['name'] for n in config_data]
  if name not in names:
    raise ValueError
  # Instantiate a Node.
  for node_data in config_data:
    if name != node_data['name']:
      continue
    # The phone_number key is optional and may not appear in the config file.
    phone_number = ''
    if 'phone_number' in node_data:
      phone_number = node_data['phone_number']
    node = Node(name=name, ip_address=node_data['ip_address'],
                sim=node_data['sim'], phone_number=phone_number)
    return node



"""The eno-python package."""

from setuptools import setup


with open('readme.md') as f:
  README = f.read()


VERSION = '0.0.5'


setup(
  name='eno',
  version=VERSION,
  description='eno test client',
  long_description=README,
  url='http://github.com/endaga/eno-python',
  download_url='https://github.com/endaga/eno-python/tarball/%s' % VERSION,
  author='Matt Ball',
  author_email='matt@endaga.com',
  license='MIT',
  packages=['eno'],
  install_requires=[
    "Adafruit-BBIO==0.0.30",
    "Flask==0.10.1",
    "Jinja2==2.8",
    "MarkupSafe==0.23",
    "PyYAML==3.11",
    "Werkzeug==0.10.4",
    "argparse==1.2.1",
    "itsdangerous==0.24",
    "pyserial==2.7",
    "python-gsmmodem==0.9",
    "requests==2.7.0",
    "wsgiref==0.1.2",
  ],
  zip_safe=False,
  scripts=[
    'scripts/run_eno_server',
  ]
)
